package hvn.agreement;

import java.util.Enumeration;

import weka.classifiers.AbstractClassifier;
import weka.core.Attribute;
import weka.core.Capabilities;
import weka.core.Capabilities.Capability;
import weka.core.Instance;
import weka.core.Instances;
import weka.core.RevisionUtils;
import weka.core.Utils;

/**
 * <!-- globalinfo-start -->
 * Class for building and using a inter-rater reliability calculator.
 * <p/>
 * <!-- globalinfo-end -->
 * 
 * @author Huy
 * @version $Revision: -1 $
 */
public class Agreement extends AbstractClassifier {

	/** for serialization */
	static final long serialVersionUID = -1;

	/** The class value 0R predicts. */
	private double m_ClassValue;

	/** The number of instances in each class (null if class numeric). */
	private double[] m_Counts;

	/** The class attribute. */
	private Attribute m_Class;
	
	/** Huy's code */
	private int m_LastAttribute;

	/**
	 * Returns a string describing classifier
	 * 
	 * @return a description suitable for displaying in the
	 *         explorer/experimenter gui
	 */
	public String globalInfo() {
		return "Class for building and using a inter-rater reliability calculator.";
	}

	/**
	 * Returns default capabilities of the classifier.
	 * 
	 * @return the capabilities of this classifier
	 */
	public Capabilities getCapabilities() {
		Capabilities result = super.getCapabilities();
		result.disableAll();

		// attributes
		result.enable(Capability.NOMINAL_ATTRIBUTES);
	    result.enable(Capability.STRING_ATTRIBUTES);

		// class
		result.enable(Capability.NOMINAL_CLASS);
	    result.enable(Capability.MISSING_CLASS_VALUES);

		// instances
		result.setMinimumNumberInstances(1);

		return result;
	}

	/**
	 * Generates the classifier.
	 * 
	 * @param instances
	 *            set of instances serving as training data
	 * @throws Exception
	 *             if the classifier has not been generated successfully
	 */
	public void buildClassifier(Instances instances) throws Exception {
		// can classifier handle the data?
		getCapabilities().testWithFail(instances);

		// remove instances with missing class
		instances = new Instances(instances);
		instances.deleteWithMissingClass();

		double sumOfWeights = 0;
		
		m_LastAttribute = instances.numAttributes() - 2;
		m_Class = instances.classAttribute();
		
		m_ClassValue = 0;
		switch (instances.classAttribute().type()) {
		case Attribute.NUMERIC:
			m_Counts = null;
			break;
		case Attribute.NOMINAL:
			m_Counts = new double[instances.numClasses()];
			for (int i = 0; i < m_Counts.length; i++) {
				m_Counts[i] = 1;
			}
			sumOfWeights = instances.numClasses();
			break;
		}
		Enumeration enu = instances.enumerateInstances();
		while (enu.hasMoreElements()) {
			Instance instance = (Instance) enu.nextElement();
			if (!instance.classIsMissing()) {
				if (instances.classAttribute().isNominal()) {
					m_Counts[(int) instance.classValue()] += instance.weight();
				} else {
					m_ClassValue += instance.weight() * instance.classValue();
				}
				sumOfWeights += instance.weight();
			}
		}
		if (instances.classAttribute().isNumeric()) {
			if (Utils.gr(sumOfWeights, 0)) {
				m_ClassValue /= sumOfWeights;
			}
		} else {
			m_ClassValue = Utils.maxIndex(m_Counts);
			Utils.normalize(m_Counts, sumOfWeights);
		}
	}

	/**
	 * Classifies a given instance.
	 * 
	 * @param instance
	 *            the instance to be classified
	 * @return index of the predicted class
	 */
	public double classifyInstance(Instance instance) {

		String classLabel = instance.stringValue(m_LastAttribute);
		m_ClassValue = instance.classAttribute().indexOfValue(classLabel);

		return m_ClassValue;
	}

	/**
	 * Returns a description of the classifier.
	 * 
	 * @return a description of the classifier as a string.
	 */
	public String toString() {

		if (m_Class == null) {
			return "RaterAgreement: No model built yet.";
		}
		if (m_Counts == null) {
			return "RaterAgreement predicts class value: " + m_ClassValue;
		} else {
			return "RaterAgreement class value: "
					+ m_Class.value((int) m_ClassValue);
		}
	}

	/**
	 * Returns the revision string.
	 * 
	 * @return the revision
	 */
	public String getRevision() {
		return RevisionUtils.extract("$Revision: -1 $");
	}

	/**
	 * Main method for testing this class.
	 * 
	 * @param argv
	 *            the options
	 */
	public static void main(String[] argv) {
		runClassifier(new Agreement(), argv);
	}
}
